**********************************************************************************************
C  File  snowdv22.f  
C  Driver program for the Utah Energy Balance
C  Snow Accumulation and Melt Model.
C  Version 2.2  
c
C  Authors
C  David G. Tarboton (dtarb@cc.usu.edu), Utah Water Research Laboratory, Utah State University
C  Charlie Luce cluce (cluce@fs.fed.us), USDA Forest Service, Rocky Mountain Research Station, Boise, ID
C  Graduate Students who have worked on this code
c    Jinsheng You, PhD, 2004 (youjins@hotmail.com)
C    Tanveer Chowdhury, MS, 1993 
C    Tom Jackson, PhD, 1994 
c
C  Last Change 1/14/05 to allow daily temperature averaging to be handled in snowueb2
c  Version 2.1 change was to accommodate glacier surface runoff.
C
C  This program was written and prepared under agreement with and funding 
C  by the U.S. Government, and therefore is in the public domain and not
C  subject to copyright.
**********************************************************************************************
     
      PARAMETER(nsv=9,npar=32,nxv=6,niv=8,nov=14) 
      CHARACTER*200 inFILE,outFILE,pfile,svfile,bcfile
      CHARACTER*200 afile
      INTEGER YEAR,DAY,iflag(6)                        ! Pass a flag to control surface temperature modeling 
      REAL IO,LAT
      real INPT(niv,1)
      real sitev(nsv)
      real outv(nov)
      real statev(nxv)
      real param(npar)
      real dtbar(12)
      real mtime(4)                                     ! YJS pass to reflect the change of Snow (Year, month, Date, Hour)
      integer istep
C  Arrays to keep records of surface temperature and snowpace average 
C  temperature. This is for the fourth model (Modified force restore approach) 

      real, allocatable :: tsprevday(:)
      real, allocatable :: taveprevday(:)

C  Add a control of ground heat input
      integer mQgOption

C  Constant data set
      data to /0.0/        !  Temperature of freezing (0 C)
      data tk /273.15/     !  Temperature to convert C to K (273.15)
      data sbc /2.0747e-7/ !  Stefan boltzman constant (2.0747e-7 KJ/m^2/hr/K)
      data hf /333.5/      !  Heat of fusion (333.5 KJ/kg)
      data hneu /2834.0/   !  Heat of Vaporization (Ice to Vapor, 2834 KJ/kg)
      data cw /4.18/       !  Water Heat Capacity (4.18 KJ/kg/C)
      data cs /2.09/       !  Ice heat capacity (2.09 KJ/kg/C)
      data cp /1.005/      !  Air Heat Capacity (1.005 KJ/kg/K)
      data Rag /287.0/     !  Ideal Gas constant for dry air (287 J/kg/K) (name changed)
      data k  /0.4/        !  Von Karmans constant (0.4)
      data hff /3600.0/    !  Factor to convert /s into /hr (3600)
      data rhoi /917.0/    !  Density of Ice (917 kg/m^3)
      data rhow /1000.0/   !  Density of Water (1000 kg/m^3)
      data g    /9.81/     !  Gravitational acceleration (9.81 m/s^2)
      data w1day /0.261799/!  Daily frequency (2pi/24 hr 0.261799 radians/hr) 
      data pi /3.141592654/!  Pi
      data Io/4914.0/      !  solar constant  Kj/m^2/hr
C  End of the constant data set declaration

C  Input Arguments
      narg=iargc()
      IF(narg .ge. 1)THEN
        CALL getarg(1,afile)
      ELSE
        WRITE(6,*) 'give filename of input files'
        READ(5,'(A200)') afile
      ENDIF

C  Open and Read Input File
      OPEN(1,FILE=afile,STATUS= 'OLD')
      READ(1,*)inFILE,OutFILE,pfile,svfile,bcfile
      READ(1,*) irad                                         ! For Radiation Options

c     Flag to control radiation
c     0 is no measurements - radiation estimated from diurnal temperature range
c     1 is incoming shortwave radiation read from file (measured), incoming longwave estimated
c     2 is incoming shortwave and longwave radiation read from file (measured)
c     3 is net radiation read from file (measured)
      CLOSE(1)


      CALL readvals(param,pfile,npar)                                  ! Parameter file
      CALL readsv(sitev,svfile,nsv,slope,azi,lat)                      ! Site variable file
      CALL bcparm(dtbar,bca,bcc,bcfile)                                ! Bristow Campbell File
      OPEN(8,FILE=infile,STATUS='OLD')
      READ(8,*)month,day,year,hour,dt,(statev(i),i=1,nxv-2), ts_last   !yjs to disable reading refreezing information
    
      READ(8,*)ntos,iflag(5),mQgOption                                 !read in skip line, the model option, and Qg modeling option

C  skip ntos time steps at the beginning of the file
      do 2 i=1,ntos
        if(mQgOption .eq. 2) then                                     !try if new ground heat input exit
          read(8,*) TA,P,V,RH,trange,QSIOBS,QNETOB,qg
        else
          read(8,*) TA,P,V,RH,trange,QSIOBS,QNETOB
        endif
        CALL UPDATETIME(YEAR,MONTH,DAY,HOUR,DT)
 2    continue

      nstepday=24/dt

      ALLOCATE(Tsprevday(nstepday))
      ALLOCATE(Taveprevday(nstepday))


C  Initialize Tsbackup and TaveBackup
      DO 3 i = 1,nstepday
		Tsprevday(i)=-9999.
		Taveprevday(i)=-9999.0
 3    CONTINUE

      IF(ts_last .le. -9999.)THEN    
        Tsprevday(nstepday)=0  
      ELSE
        Tsprevday(nstepday)=ts_last                      !has measurements

      END IF 

******************* To compute Ave.Temp (For Previous day temp.)  ******************************

	Us   = statev(1)                ! Ub in UEB
      Ws   = statev(2)			    ! W in UEB
	Wc   = statev(4)
	Apr  = sitev(2)				    ! Atm. Pressure  (PR in UEB)
      cg   = param(4)					! Ground heat capacity (nominally 2.09 KJ/kg/C)
      rhog = param(8)					! Soil Density (nominally 1700 kg/m^3)
      de   = param(11)				! Thermally active depth of soil (0.1 m)
    
	Tave = TAVG(Us,Ws,RHOW,CS,TO,RHOG,DE,CG,HF)        ! This call only
      
      Taveprevday(nstepday) = Tave

*************************************************************************************************     
      OPEN(9,FILE=OUTFILE,STATUS='UNKNOWN')

c   initialize variables for mass balance
      Ws1   = statev(2)
	Wc1   = statev(4)
      cumP  = 0.
      cumEs  = 0.
      cumEc = 0.
      cumMr = 0.
*	CumMc = 0. 
       istep =1    
C******************  Main Loop Starts Here.  Exits upon end of INPT file ************************

1	READ(8,*,END =99) TA,P,V,RH,trange,QSIOBS,QNETOB,qg, qli         !(Ws=V, Changed)

       if(mqgoption .eq. 2)then 
         sitev(3)=qg
       endif
        
       INPT(1,1)=TA
       INPT(2,1)=P
       INPT(3,1)=V
       INPT(4,1)=RH
       INPT(7,1) = QNETOB


	 

******************     Radiation Input Parameterization  ***************************************
      
       CALL hyri(YEAR,MONTH,DAY,HOUR,DT,SLOPE,AZI,LAT,HRI,COSZEN)

       INPT(8,1)=COSZEN                
       IF(IRAD.LE.2)THEN               
            CALL atf(atff,trange,month,dtbar,bca,bcc)
            IF(IRAD.EQ.0) THEN                     
                INPT(5,1)=atff*IO*HRI
    			  CALL cloud(param,atff,cf)   ! For cloudiness fraction
            ELSE 
c      Need to call HYRI for horizontal surface to perform horizontal
C      measurement adjustment
               CALL hyri(YEAR,MONTH,DAY,HOUR,DT,0.0,AZI,LAT,HRI0,COSZEN)
c      If HRI0 is 0 the sun should have set so QSIOBS should be 0.  If it is
C      not it indicates a potential measurement problem. i.e. moonshine
                 if(HRI0 .GT. 0.0) then
                    atfimplied=min(qsiobs/(HRI0*IO),0.9) ! To avoid unreasonably large radiation when HRI0 is small
                    INPT(5,1)=atfimplied * HRI * IO
                 else
                   INPT(5,1)=QSIOBS
                     if(qsiobs .ne. 0.)then
                        write(6,*)"Warning ! you have nonzero nightime"
                        write(6,*)"incident radiation of",qsiobs
                        write(6,*)"at date",year,month,day,hour
                     endif
                 endif
               cf = 1.-atff/bca          ! Cf based on solar radiation measurement
            ENDIF
             
		  IF(irad .lt. 2)THEN    
                CALL qlif(TA,RH,TK,SBC,Ema,Eacl,cf,inpt(6,1) )
		  Else
				inpt(6,1)=qli
            ENDIF 

          IRADFL=0                        ! Long wave or shortwave either measured and calculated

       ELSE 
              IRADFL=1                    ! This case is when given IRAD =3 (From Net Radiation)  
              INPT(7,1) = QNETOB          
       ENDIF

************************************************************************************************

      CALL UPDATETIME(YEAR,MONTH,DAY,HOUR,DT)
      WRITE(9,*)YEAR,MONTH,DAY,HOUR,atff,HRI,
     *	Eacl,Ema,(INPT(i,1),i=1,8)                 

C  set control flags
       iflag(1) = iradfl   ! radiation (0=radiation is shortwave in col 5 and longwave in col 6, else = net radiation in column 7)
       iflag(2) = 1        ! Printing
       iflag(3) = 9        ! Output unit
       iflag(4) = 1        ! Albedo calculation
c      iflag(5) =          ! yjs Surface temperature modeling method, read above from input file
       iflag(6) = 1        ! Wind speed in forest canopy: 1 is observed and 2 is computed from above canopy observation
c  set modeling time
       mtime(1) = Year
       mtime(2) = month
       mtime(3) = day
       mtime(4) = hour

**************************************************************************************************
	CALL SNOWUEB2(dt,1,inpt,sitev,statev,
     &   tsprevday, taveprevday, nstepday, param,iflag,  
     &   cump,cumes,cumEc,cummr,outv,mtime
     &  ,atff,cf)
                     
*************************************************************************************************    

       errmbal=Ws1-statev(2)+Wc1-statev(4)+ cump-cumMr-cumEs-cumEc         ! changed vinod cum Mc??
       WRITE(9,*)ERRMBAL

cyjs Try to model the ground heat flux if the qg option is set to 3. 02/24/01
       if(mQgOption .eq. 3) then
          qg=param(16)*(param(20)-outv(11))/param(11)
          sitev(3) =qg
       endif
	istep= istep+1             !  
       go to 1

C      MASS BALANCE ERROR

 99    WRITE(6,*)'TOTAL MASS BALANCE ERROR =',ERRMBAL
       close(8)
       close(9)
       STOP
       END



*************************************************************************************************